@@ -1,12 +1,15 @@
+"""The main entry point for running ExplainaBoard."""
+
 from __future__ import annotations
 
 import argparse
+import copy
 import json
 import os
 
 import eaas.endpoint
 
-from explainaboard import get_loader_class, get_processor, TaskType
+from explainaboard import get_loader_class, get_processor_class, TaskType
 from explainaboard.constants import Source
 from explainaboard.info import SysOutputInfo
 from explainaboard.loaders.file_loader import (
@@ -15,8 +18,8 @@ from explainaboard.loaders.file_loader import (
     FileLoaderMetadata,
 )
 from explainaboard.metrics.eaas import EaaSMetricConfig
-from explainaboard.metrics.metric import MetricConfig
-from explainaboard.metrics.registry import get_metric_config_class
+from explainaboard.metrics.metric import MetricConfig, Score
+from explainaboard.serialization import common_registry
 from explainaboard.utils.io_utils import text_writer
 from explainaboard.utils.logging import get_logger
 from explainaboard.utils.tensor_analysis import (
@@ -26,31 +29,34 @@ from explainaboard.utils.tensor_analysis import (
 )
 from explainaboard.utils.typing_utils import unwrap
 from explainaboard.visualizers import get_pairwise_performance_gap
-from explainaboard.visualizers.draw_hist import draw_bar_chart_from_reports
+from explainaboard.visualizers.draw_charts import draw_charts_from_reports
 
 
 def get_tasks(task: TaskType, system_outputs: list[str]) -> list[TaskType]:
-    """
-    Get the task for each system output.
-    :param task: Explicitly specified task. Use if present
-    :param system_outputs: System output files, load from metadata in these files if
-      an explicit task is not set
-    :return: A list of task types for each system
+    """Get the task for each system output.
+
+    Args:
+        task: Explicitly specified task. Use if present
+        system_outputs: System output files, load from metadata in these files if
+          an explicit task is not set
+
+    Returns:
+        A list of task types for each system
     """
     real_tasks: list[TaskType] = []
     if task:
         real_tasks = [task] * len(system_outputs)
         if task not in TaskType.list():
             raise ValueError(
-                f'Task name {task} was not recognized. ExplainaBoard currently '
-                f'supports:{TaskType.list()}'
+                f"Task name {task} was not recognized. ExplainaBoard currently "
+                f"supports:{TaskType.list()}"
             )
         return real_tasks
     else:
         for sys_output in system_outputs:
             # give me a task, or give me death (by exception)
             task_or_die: TaskType | None = None
-            msg: str = ''
+            msg: str = ""
             try:
                 metadata = FileLoaderMetadata.from_file(sys_output)
                 task_or_die = TaskType(unwrap(metadata.task_name))
@@ -58,16 +64,17 @@ def get_tasks(task: TaskType, system_outputs: list[str]) -> list[TaskType]:
                 msg = str(e)
             if task_or_die is None:
                 raise ValueError(
-                    'Must either specify a task explicitly or have one '
-                    'specified in metadata, but could find neither for '
-                    f'{sys_output}. {msg}'
+                    "Must either specify a task explicitly or have one "
+                    "specified in metadata, but could find neither for "
+                    f"{sys_output}. {msg}"
                 )
             real_tasks.append(unwrap(task_or_die))
     return real_tasks
 
 
 def analyze_reports(args):
-    """
+    """Analyze reports based on the input arguments.
+
     score_tensor is a nested dict, for example
     score_tensor[system_name][dataset_name][language] =
     {
@@ -122,10 +129,15 @@ def analyze_reports(args):
 
 
 def create_parser():
-    parser = argparse.ArgumentParser(description='Explainable Leaderboards for NLP')
-    parser.add_argument('--task', type=str, required=False, help="the task name")
+    """Create the parser with argparse.
+
+    Returns:
+        The parser.
+    """
+    parser = argparse.ArgumentParser(description="Explainable Leaderboards for NLP")
+    parser.add_argument("--task", type=str, required=False, help="the task name")
     parser.add_argument(
-        '--system_outputs',
+        "--system-outputs",
         type=str,
         required=True,
         nargs="+",
@@ -136,7 +148,7 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--reports',
+        "--reports",
         type=str,
         required=False,
         nargs="+",
@@ -145,7 +157,7 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--systems',
+        "--systems",
         type=str,
         required=False,
         nargs="+",
@@ -153,7 +165,7 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--datasets',
+        "--datasets",
         type=str,
         required=False,
         nargs="+",
@@ -161,7 +173,7 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--languages',
+        "--languages",
         type=str,
         required=False,
         nargs="+",
@@ -169,28 +181,28 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--systems_aggregation',
+        "--systems-aggregation",
         type=str,
         required=False,
         help="None|minus|combination",
     )
 
     parser.add_argument(
-        '--datasets_aggregation',
+        "--datasets-aggregation",
         type=str,
         required=False,
         help="None|average|",
     )
 
     parser.add_argument(
-        '--languages_aggregation',
+        "--languages-aggregation",
         type=str,
         required=False,
         help="None|average|",
     )
 
     parser.add_argument(
-        '--dataset',
+        "--dataset",
         type=str,
         required=False,
         default=None,
@@ -198,7 +210,7 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--sub_dataset',
+        "--sub-dataset",
         type=str,
         required=False,
         default=None,
@@ -206,17 +218,17 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--split',
+        "--split",
         type=str,
         required=False,
-        default='test',
+        default="test",
         help="the name of the split within the dataset",
     )
 
     parser.add_argument(
-        '--language',
-        '--target_language',
-        dest='target_language',
+        "--language",
+        "--target-language",
+        dest="target_language",
         type=str,
         required=False,
         default=None,
@@ -225,7 +237,7 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--source_language',
+        "--source-language",
         type=str,
         required=False,
         default=None,
@@ -233,15 +245,14 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--reload_stat',
-        type=str,
-        required=False,
-        default=None,
-        help="reload precomputed statistics over training set (if exists)",
+        "--no-use-cache",
+        dest="use_cache",
+        action="store_false",
+        help="Disable cached statistics over training set.",
     )
 
     parser.add_argument(
-        '--metrics',
+        "--metrics",
         type=str,
         required=False,
         nargs="*",
@@ -249,7 +260,7 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--output_file_type',
+        "--output-file-type",
         type=str,
         required=False,
         default=None,
@@ -257,15 +268,26 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--conf_value',
+        "--conf-value",
+        dest="confidence_alpha",
         type=float,
         required=False,
-        default=0.05,
-        help="the p-value with which to calculate the confidence interval",
+        help="Deprecated. use --confidence-alpha instead.",
     )
 
     parser.add_argument(
-        '--output_dir',
+        "--confidence-alpha",
+        type=float,
+        required=False,
+        default=0.05,
+        help=(
+            "the *inverse* confidence level of confidence intervals. If you need to "
+            "set the confidence level to 0.95, set this value to 0.05."
+        ),
+    )
+
+    parser.add_argument(
+        "--output-dir",
         type=str,
         required=False,
         default=None,
@@ -273,7 +295,7 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--report_json',
+        "--report-json",
         type=str,
         required=False,
         default=None,
@@ -281,24 +303,30 @@ def create_parser():
     )
 
     parser.add_argument(
-        '--system_details',
+        "--system-details",
         type=str,
         required=False,
         help="a json file to store detailed information for a system",
     )
 
     parser.add_argument(
-        '--custom_dataset_paths',
+        "--custom-dataset-paths",
         type=str,
         nargs="*",
         help="path to custom dataset",
     )
 
     parser.add_argument(
-        '--custom_dataset_file_type',
+        "--custom-dataset-file-type",
         type=str,
         help="file types for custom datasets",
     )
+
+    parser.add_argument(
+        "--skip-failed-analyses",
+        action="store_true",
+        help="whether to skip failed analyses or report errors.",
+    )
     return parser
 
 
@@ -315,7 +343,10 @@ def get_metric_config_or_eaas(name: str) -> type[MetricConfig]:
         ValueError: `name` is not registered in neither the registry nor EaaS.
     """
     try:
-        return get_metric_config_class(name)
+        cls = common_registry.get_type(name)
+        if not issubclass(cls, MetricConfig):
+            raise TypeError(f"Obtained class is not a MetricConfig: {cls.__name__}")
+        return cls
     except ValueError:
         if name in eaas.endpoint.EndpointConfig().valid_metrics:
             return EaaSMetricConfig
@@ -327,9 +358,10 @@ def get_metric_config_or_eaas(name: str) -> type[MetricConfig]:
 
 # TODO(Pengfei): The overall implementation of this script should be deduplicated
 def main():
+    """The main function to be executed."""
     args = create_parser().parse_args()
 
-    reload_stat: bool = False if args.reload_stat == "0" else True
+    use_cache: bool = args.use_cache
     system_outputs: list[str] = args.system_outputs
 
     reports: list[str] | None = args.reports
@@ -350,7 +382,7 @@ def main():
                     with open(args.system_details) as fin:
                         return json.load(fin)
                 except ValueError as e:
-                    raise ValueError(f'invalid json: {e} for system details')
+                    raise ValueError(f"invalid json: {e} for system details")
 
         output_dir_figures = os.path.join(output_dir, "figures") if output_dir else None
         output_dir_reports = os.path.join(output_dir, "reports") if output_dir else None
@@ -363,7 +395,7 @@ def main():
         if output_dir_reports and not os.path.exists(output_dir_reports):
             os.makedirs(output_dir_reports)
 
-        # check for benchmark submission: explainaboard  --system_outputs ./data/
+        # check for benchmark submission: explainaboard  --system-outputs ./data/
         # system_outputs/sst2/user_specified_metadata.json
         num_systems = len(system_outputs)
         dataset_file_types: list[str | None] = [dataset_file_type] * num_systems
@@ -423,15 +455,15 @@ def main():
                 num0 = len(system_datasets[0])
                 num1 = len(system_datasets[1])
                 raise ValueError(
-                    f'Data must be identical for pairwise analysis, but length of '
-                    'files '
-                    f'{system_datasets[0]} ({num0}) != {system_datasets[1]} ({num1})'
+                    f"Data must be identical for pairwise analysis, but length of "
+                    "files "
+                    f"{system_datasets[0]} ({num0}) != {system_datasets[1]} ({num1})"
                 )
 
         # TODO(gneubig): This gets metadata from the first system and assumes it's the
         #  same for other systems
         target_language = (
-            target_language or system_datasets[0].metadata.target_language or 'en'
+            target_language or system_datasets[0].metadata.target_language or "en"
         )
         source_language = (
             source_language
@@ -446,19 +478,21 @@ def main():
             "split_name": split,
             "source_language": source_language,
             "target_language": target_language,
-            "reload_stat": reload_stat,
-            "conf_value": args.conf_value,
+            "confidence_alpha": args.confidence_alpha,
             "system_details": system_details,
             "custom_features": system_datasets[0].metadata.custom_features,
+            "custom_analyses": system_datasets[0].metadata.custom_analyses,
         }
-
         if metric_names is not None:
-            if 'metric_configs' in metadata:
-                raise ValueError('Cannot specify both metric names and metric configs')
-            metric_configs = [
-                get_metric_config_or_eaas(name)(name, source_language, target_language)
+            if "metric_configs" in metadata:
+                raise ValueError("Cannot specify both metric names and metric configs")
+            metric_configs = {
+                name: get_metric_config_or_eaas(name)(
+                    source_language=source_language,
+                    target_language=target_language,
+                )
                 for name in metric_names
-            ]
+            }
             metadata["metric_configs"] = metric_configs
 
         # Run analysis
@@ -466,31 +500,31 @@ def main():
         for loader, system_dataset, system_full_path, task in zip(
             loaders, system_datasets, system_outputs, tasks
         ):
+            metadata_copied = copy.deepcopy(metadata)
+            metadata_copied["task_name"] = task
 
-            # metadata.update(loader.user_defined_metadata_configs)
-            # metadata[
-            #     "user_defined_features_configs"
-            # ] = loader.user_defined_features_configs
-            metadata["task_name"] = task
-
-            processor = get_processor(task=task)
+            processor = get_processor_class(task=task)()
             report = processor.process(
-                metadata=metadata, sys_output=system_dataset.samples
+                metadata=metadata_copied,
+                sys_output=system_dataset.samples,
+                skip_failed_analyses=args.skip_failed_analyses,
+                use_cache=use_cache,
             )
             reports.append(report)
 
             # print to the console
-            get_logger('report').info('--- Overall Performance')
-            for overall_level in report.results.overall:
-                for metric_stat in overall_level:
-                    get_logger('report').info(
-                        f'{metric_stat.metric_name}\t{metric_stat.value}'
-                    )
-            get_logger('report').info('')
-            get_logger('report').info('--- Fine-grained Analyses')
+            logger = get_logger("report")
+
+            logger.info("--- Overall Performance")
+            for level_name, overall in report.results.overall.items():
+                for metric_name, metric_result in overall.items():
+                    value = metric_result.get_value(Score, "score").value
+                    logger.info(f"{level_name}\t{metric_name}\t{value}")
+            logger.info("")
+            logger.info("--- Fine-grained Analyses")
             for analysis in report.results.analyses:
                 if analysis is not None:
-                    analysis.print()
+                    logger.info(analysis.generate_report())
 
             if output_dir:
 
@@ -501,7 +535,7 @@ def main():
                 # generate figures and save them into  `output_dir_figures`
                 if not os.path.exists(f"{output_dir_figures}/{x_file_name}"):
                     os.makedirs(f"{output_dir_figures}/{x_file_name}")
-                draw_bar_chart_from_reports(
+                draw_charts_from_reports(
                     [f"{output_dir_reports}/{x_file_name}.json"],
                     f"{output_dir_figures}/{x_file_name}",
                 )
@@ -514,5 +548,5 @@ def main():
                 compare_analysis.print_as_json(file=report_file)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
