@@ -1,6 +1,8 @@
+"""Loaders for conditional generation tasks."""
+
 from __future__ import annotations
 
-from explainaboard.constants import FileType, TaskType
+from explainaboard.constants import FileType
 from explainaboard.loaders.file_loader import (
     DatalabFileLoader,
     FileLoader,
@@ -10,33 +12,30 @@ from explainaboard.loaders.file_loader import (
     TSVFileLoader,
 )
 from explainaboard.loaders.loader import Loader
-from explainaboard.loaders.loader_registry import register_loader
 
 
-@register_loader(TaskType.conditional_generation)
-@register_loader(TaskType.machine_translation)
 class ConditionalGenerationLoader(Loader):
-    """
-    Validate and Reformat system output file with tsv format:
-    text \t true_label \t predicted_label
+    """Loader for the conditional generation task.
 
     usage:
-        please refer to `test_loaders.py`
+        please refer to `loaders_test.py`
     """
 
-    OUTPUT_FIELDS = ['source', 'reference']
-    JSON_FIELDS: list[str | tuple[str, str]] = ['source', 'reference']
+    OUTPUT_FIELDS = ["source", "reference"]
+    JSON_FIELDS: list[str | tuple[str, str]] = ["source", "reference"]
     JSON_FIELDS_DATALAB: list[str | tuple[str, str]] = [
-        'source_column',
-        'reference_column',
+        "source_column",
+        "reference_column",
     ]
 
     @classmethod
     def default_dataset_file_type(cls) -> FileType:
+        """See Loader.default_dataset_file_type."""
         return FileType.tsv
 
     @classmethod
     def default_dataset_file_loaders(cls) -> dict[FileType, FileLoader]:
+        """See Loader.default_dataset_file_loaders."""
         return {
             FileType.tsv: TSVFileLoader(
                 [
@@ -64,6 +63,7 @@ class ConditionalGenerationLoader(Loader):
 
     @classmethod
     def default_output_file_loaders(cls) -> dict[FileType, FileLoader]:
+        """See Loader.default_output_file_loaders."""
         field_name = "hypothesis"
         return {
             FileType.text: TextFileLoader(field_name, str),
@@ -73,22 +73,24 @@ class ConditionalGenerationLoader(Loader):
         }
 
 
-@register_loader(TaskType.summarization)
 class SummarizationLoader(ConditionalGenerationLoader):
+    """A loader for summarization."""
+
     JSON_FIELDS_DATALAB: list[str | tuple[str, str]] = [
-        'source_column',
-        'reference_column',
+        "source_column",
+        "reference_column",
     ]
 
 
-@register_loader(TaskType.machine_translation)
 class MachineTranslationLoader(ConditionalGenerationLoader):
+    """A loader for machine translation."""
+
     JSON_FIELDS_DATALAB = [
-        ('translation', FileLoaderField.SOURCE_LANGUAGE),
-        ('translation', FileLoaderField.TARGET_LANGUAGE),
+        ("translation", FileLoaderField.SOURCE_LANGUAGE),
+        ("translation", FileLoaderField.TARGET_LANGUAGE),
     ]
 
     JSON_FIELDS = [
-        ('translation', FileLoaderField.SOURCE_LANGUAGE),
-        ('translation', FileLoaderField.TARGET_LANGUAGE),
+        ("translation", FileLoaderField.SOURCE_LANGUAGE),
+        ("translation", FileLoaderField.TARGET_LANGUAGE),
     ]
